package com.example.demo;

import java.util.Date;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.demo.comments.model.CommentEntity;
import com.example.demo.comments.service.CommentService;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.users.model.UserEntity;
import com.example.demo.posts.model.PostEntity;
import com.example.demo.users.service.UserService;
import com.example.demo.posts.service.PostService;;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class CommentServiceTests {
    @Autowired
    private UserService userService;
    @Autowired
    private PostService postService;
    @Autowired
    private CommentService commentService;

    private CommentEntity comment;

    @Test
    void createNullableTest() {
        final CommentEntity nullableType = new CommentEntity(null, null, null, null);
        Assertions.assertThrows(DataIntegrityViolationException.class, () -> commentService.create(nullableType));
    }

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> commentService.get(0L));
    }

    @BeforeEach
    void createData() {
        removeData();

        var user1 = userService.create(new UserEntity("sardq", "as2@asd", "asdsd", null));
        var user2 = userService.create(new UserEntity("eafqr", "aasd@asd", "1234r", null));
        var user3 = userService.create(new UserEntity("asf", "aaasfsfs@asd", "332a", null));
        var post1 = postService.create(new PostEntity(user1, new Date(), "test",
                Long.valueOf(0), Long.valueOf(0)));
        var post2 = postService
                .create(new PostEntity(user3, new Date(), "asfgczv", Long.valueOf(20),
                        Long.valueOf(15)));
        var post3 = postService.create(new PostEntity(user2, new Date(), "asvbsfd", Long.valueOf(30),
                Long.valueOf(3)));
        commentService.create(new CommentEntity(user1, post2, new Date(), "my comment"));
        commentService.create(new CommentEntity(user3, post1, new Date(), "my comment2"));
        comment = commentService
                .create(new CommentEntity(user2, post3, new Date(), "my comment3"));
    }

    @AfterEach
    void removeData() {
        commentService.getAll().forEach(item -> commentService.delete(item.getId()));
        postService.getAll().forEach(item -> postService.delete(item.getId()));
        userService.getAll().forEach(item -> userService.delete(item.getId()));
    }

    @Test
    void createTest() {
        Assertions.assertEquals(3, commentService.getAll().size());
    }

    @Test
    void updateTest() {
        final String test = "TEST";
        final CommentEntity entity = commentService.get(comment.getId());
        final String oldText = entity.getText();
        var user1 = userService.create(new UserEntity("sardq", "asfa2@asd", "asdsd", null));
        var post1 = postService.create(new PostEntity(user1,
                new Date(), "test",
                Long.valueOf(0), Long.valueOf(0)));

        final CommentEntity newEntity = commentService.update(comment
                .getId(),
                new CommentEntity(user1, post1, new Date(), test));
        Assertions.assertEquals(3, commentService.getAll().size());
        Assertions.assertEquals(test, newEntity.getText());
        Assertions.assertNotEquals(oldText, newEntity.getText());
    }

    @Test
    void deleteTest() {
        commentService.delete(comment.getId());
        Assertions.assertEquals(2, commentService.getAll().size());

        final var user1 = userService.create(new UserEntity("sardq", "asfa2@asd", "asdsd", null));
        final var post1 = postService.create(new PostEntity(user1,
                new Date(), "test",
                Long.valueOf(0), Long.valueOf(0)));
        commentService.create(
                new CommentEntity(user1, post1, new Date(), "jgdssd"));
        Assertions.assertEquals(3, commentService.getAll().size());
    }
}
