package com.example.demo;

import java.util.Date;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.users.model.UserEntity;
import com.example.demo.posts.model.PostEntity;
import com.example.demo.users.service.UserService;

import jakarta.transaction.Transactional;

import com.example.demo.posts.service.PostService;;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class PostServiceTests {
    @Autowired
    private UserService userService;
    @Autowired
    private PostService postService;

    private PostEntity post;

    @BeforeEach
    void createData() {
        removeData();

        var user1 = userService.create(new UserEntity("sardq", "aqqq2@asd", "asdsd", null));
        var user2 = userService.create(new UserEntity("eafqr", "qqqq@asd", "1234r", null));
        var user3 = userService.create(new UserEntity("asf", "wwww@asd", "332a", null));
        postService.create(new PostEntity(user1,
                new Date(), "test",
                Long.valueOf(0), Long.valueOf(0)));
        postService.create(new PostEntity(user3, new Date(), "asfgczv", Long.valueOf(20), Long.valueOf(15)));
        post = postService
                .create(new PostEntity(user2,
                        new Date(), "asvbsfd", Long.valueOf(0),
                        Long.valueOf(0)));
    }

    @Test
    void createNullableTest() {
        final PostEntity nullableType = new PostEntity(null, null, null, null, null);
        Assertions.assertThrows(DataIntegrityViolationException.class, () -> postService.create(nullableType));
    }

    @Test
    @Transactional
    void ViewLikeTest() {
        postService.addLike(post.getId());
        postService.addLike(post.getId());
        postService.addLike(post.getId());
        Assertions.assertEquals(3, post.getLiked());
        postService.removeLiked(post.getId());
        postService.removeLiked(post.getId());
        Assertions.assertEquals(1, post.getLiked());
        postService.addView(post.getId());
        postService.addView(post.getId());
        Assertions.assertEquals(2, post.getViewed());

    }

    @AfterEach
    void removeData() {
        postService.getAll().forEach(item -> postService.delete(item.getId()));
        userService.getAll().forEach(item -> userService.delete(item.getId()));
    }

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> postService.get(0L));
    }

    @Test
    @Order(1)
    void createTest() {

        Assertions.assertEquals(3, postService.getAll().size());
    }

    @Test
    @Order(2)
    void updateTest() {
        final String test = "TEST";
        final String oldText = post.getText();
        final PostEntity newEntity = postService.update(post.getId(),
                new PostEntity(new UserEntity("safsaf", "eee@asd", "qazq", null),
                        new Date(), test, Long.valueOf(30), Long.valueOf(3)));
        Assertions.assertEquals(3, postService.getAll().size());
        Assertions.assertEquals(test, newEntity.getText());
        Assertions.assertNotEquals(oldText, newEntity.getText());
    }

    @Test
    @Order(3)
    void deleteTest() {
        postService.delete(post.getId());
        Assertions.assertEquals(2, postService.getAll().size());
        var user1 = userService.create(new UserEntity("sardq", "sssss@asd", "asdsd", null));
        postService.create(
                new PostEntity(user1, new Date(), "test", Long.valueOf(0), Long.valueOf(0)));
        Assertions.assertEquals(3, postService.getAll().size());
    }
}
