package com.example.demo;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

import jakarta.transaction.Transactional;

@SpringBootTest(properties = "spring.main.lazy-initialization=true")
class UserServiceTests {
    @Autowired
    private UserService userService;

    private UserEntity user;
    private UserEntity last;

    @BeforeEach
    void createData() {
        removeData();
        user = userService.create(new UserEntity("sardq", "test", "test", null));
        userService.create(new UserEntity("asd", "test2", "test1", null));
        last = userService.create(new UserEntity("asfkasf", "test3", "test2", null));
    }

    @AfterEach
    void removeData() {
        userService.getAll().forEach(item -> userService.delete(item.getId()));
    }

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> userService.get(0L));
    }

    @Test
    void createTest() {
        Assertions.assertEquals(3, userService.getAll().size());
    }

    @Test
    void createNotUniqueTest() {
        final UserEntity nonUniqueType = new UserEntity("asd", "test", "asdad", null);
        Assertions.assertThrows(IllegalArgumentException.class, () -> userService.create(nonUniqueType));
    }

    @Test
    @Transactional
    void FriendsTest() {
        userService.addFriend(user.getId(), last);
        Assertions.assertEquals(1, user.getFriends().size());
        userService.removeFriend(user.getId(), last);
        Assertions.assertEquals(0, user.getFriends().size());
    }

    @Test
    void updateTest() {
        final String test = "TEST";
        final UserEntity entity = userService.get(user.getId());
        final String oldName = entity.getNickname();
        final UserEntity newEntity = userService.update(user.getId(), new UserEntity(test, "aasd@asd", "qazq", null));
        Assertions.assertEquals(3, userService.getAll().size());
        Assertions.assertEquals(test, newEntity.getNickname());
        Assertions.assertNotEquals(oldName, newEntity.getNickname());
    }

    @Test
    void createNullableTest() {
        final UserEntity nullableType = new UserEntity(null, null, null, null);
        Assertions.assertThrows(DataIntegrityViolationException.class, () -> userService.create(nullableType));
    }

    @Test
    void deleteTest() {
        userService.delete(last.getId());
        Assertions.assertEquals(2, userService.getAll().size());

        final UserEntity newEntity = userService.create(new UserEntity("asfzxv", "aasd@asd", "qwtg", null));
        Assertions.assertEquals(3, userService.getAll().size());
        Assertions.assertNotEquals(user.getId(), newEntity.getId());
    }
}
