package com.example.demo.posts.api;

import java.util.Map;

import org.modelmapper.ModelMapper;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import java.sql.Time;

import com.example.demo.comments.api.CommentDto;
import com.example.demo.comments.api.PostCommentDto;
import com.example.demo.comments.model.CommentEntity;
import com.example.demo.comments.service.CommentService;
import com.example.demo.core.convert;
import com.example.demo.core.api.PageAttributesMapper;
import com.example.demo.core.configuration.Constants;
import com.example.demo.core.security.UserPrincipal;
import com.example.demo.posts.model.PostEntity;
import com.example.demo.posts.service.PostService;
import com.example.demo.users.service.UserService;

import jakarta.validation.Valid;

@Controller
public class HomeController {
    private static final String HOME_VIEW = "/";
    private static final String PAGE_ATTRIBUTE = "page";
    private static final String COMMENT_ATTRIBUTE = "comment";
    private static final String POST_EDIT_VIEW = "post-edit";
    private static final String COMMENT_EDIT_VIEW = "comment-edit";

    private static final String POST_ATTRIBUTE = "post";
    private static final String COMMENT_VIEW = "comments";

    private final UserService userService;
    private final CommentService commentService;

    private final PostService postService;
    private final ModelMapper modelMapper;

    public HomeController(PostService postService, UserService userService, ModelMapper modelMapper,
            CommentService commentService) {
        this.commentService = commentService;
        this.postService = postService;
        this.userService = userService;
        this.modelMapper = modelMapper;
    }

    private HomePostDto toDto(PostEntity entity) {
        var model = modelMapper.map(entity, HomePostDto.class);
        model.setPersonUsername(entity.getPerson().getUsername());
        convert time = new convert();
        model.setTime(time.dateToString(entity.getTime()));
        return model;
    }

    private PostCommentDto toCommentDto(CommentEntity entity) {
        var model = modelMapper.map(entity, PostCommentDto.class);
        model.setPersonUsername(entity.getperson().getUsername());
        convert time = new convert();
        model.setTime(time.dateToString(entity.getTime()));
        return model;
    }

    private PostEntity toEntity(HomePostDto dto) {
        final PostEntity entity = modelMapper.map(dto, PostEntity.class);
        entity.setPerson(userService.get(dto.getPersonId()));
        return entity;

    }

    private CommentEntity toCommentEntity(PostCommentDto dto) {
        final CommentEntity entity = modelMapper.map(dto, CommentEntity.class);
        entity.setperson(userService.get(dto.getPersonId()));
        entity.setPost(postService.get(dto.getPostId()));
        return entity;
    }

    @GetMapping
    public String getAll(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @AuthenticationPrincipal UserPrincipal principal,
            Model model) {
        final Map<String, Object> attributes = PageAttributesMapper.toAttributes(
                postService.getAll(page, Constants.DEFUALT_PAGE_SIZE), this::toDto);
        model.addAttribute("user", principal);
        model.addAllAttributes(attributes);
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return "home";
    }

    @GetMapping("/edit/")
    public String createPost(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        model.addAttribute(POST_ATTRIBUTE, new HomePostDto());
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return POST_EDIT_VIEW;
    }

    @PostMapping("/edit/")
    public String createPost(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = POST_ATTRIBUTE) @Valid HomePostDto post,
            @AuthenticationPrincipal UserPrincipal principal,
            BindingResult bindingResult,
            Model model,
            RedirectAttributes redirectAttributes) {
        if (post.getText().length() < 4) {
            bindingResult.rejectValue("text", "posts:texts", "Неправильный текс.");
            model.addAttribute(POST_ATTRIBUTE, post);
            return POST_EDIT_VIEW;
        }
        post.setPersonId(principal.getId());
        post.setPersonUsername(principal.getUsername());
        post.setLiked(0L);
        post.setViewed(0L);
        convert conv = new convert();
        post.setTime(conv.dateToFormatDTOString(new Time(System.currentTimeMillis())));
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        postService.create(toEntity(post));
        return Constants.REDIRECT_VIEW + HOME_VIEW;
    }

    @PostMapping("/addLike/{id}")
    public String addLike(@PathVariable(name = "id") Long id) {
        toDto(postService.addLike(id));
        return Constants.REDIRECT_VIEW + HOME_VIEW;
    }

    @PostMapping("/remLike/{id}")
    public String removeLike(@PathVariable(name = "id") Long id) {
        toDto(postService.removeLiked(id));
        return Constants.REDIRECT_VIEW + HOME_VIEW;
    }

    @PostMapping("/view/{id}")
    public HomePostDto addView(@PathVariable(name = "id") Long id) {
        return toDto(postService.addView(id));
    }

    @GetMapping("/edit/{id}")
    public String updatePost(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @AuthenticationPrincipal UserPrincipal principal,
            Model model) {
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        if (principal.getId() != postService.get(id).getPerson().getId()) {
            return Constants.REDIRECT_VIEW + HOME_VIEW;
        }
        model.addAttribute(POST_ATTRIBUTE, toDto(postService.get(id)));
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return POST_EDIT_VIEW;
    }

    @PostMapping("/edit/{id}")
    public String updatePost(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = POST_ATTRIBUTE) @Valid HomePostDto post,
            BindingResult bindingResult,
            @AuthenticationPrincipal UserPrincipal principal,
            Model model,
            RedirectAttributes redirectAttributes) {
        if (bindingResult.hasErrors()) {
            model.addAttribute(PAGE_ATTRIBUTE, page);
            return POST_EDIT_VIEW;
        }
        convert conv = new convert();
        post.setTime(conv.dateToFormatDTOString(new Time(System.currentTimeMillis())));
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        post.setPersonId(principal.getId());
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        postService.update(id, toEntity(post));
        return Constants.REDIRECT_VIEW + HOME_VIEW;
    }

    @PostMapping("/delete/{id}")
    public String deletePost(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @AuthenticationPrincipal UserPrincipal principal,
            RedirectAttributes redirectAttributes) {
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        if (postService.get(id).getPerson().getId() == principal.getId())
            postService.delete(id);
        return Constants.REDIRECT_VIEW + HOME_VIEW;
    }

    @GetMapping("/{id}/comments")
    public String getPostComments(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @AuthenticationPrincipal UserPrincipal principal,
            Model model) {
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        Map<String, Object> attributes = PageAttributesMapper
                .toAttributes(commentService.getAll(id, page, 5),
                        this::toCommentDto);
        model.addAllAttributes(attributes);
        model.addAttribute("post", toDto(postService.get(id)));
        var newComment = new CommentDto();
        newComment.setPostId(id);
        model.addAttribute("comment", newComment);
        model.addAttribute("user", principal);
        return COMMENT_VIEW;
    }

    @PostMapping("{id}/comments/edit")
    public String createPostComment(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = COMMENT_ATTRIBUTE) @Valid PostCommentDto comment,
            @AuthenticationPrincipal UserPrincipal principal,
            BindingResult bindingResult,
            Model model,
            RedirectAttributes redirectAttributes) {
        if (comment.getText().length() < 4) {
            bindingResult.rejectValue("text", "comments:texts", "Введите больше символов.");
            model.addAttribute(COMMENT_ATTRIBUTE, comment);
            return COMMENT_EDIT_VIEW;
        }
        comment.setPersonId(principal.getId());
        comment.setPersonUsername(principal.getUsername());
        comment.setPostId(id);
        convert conv = new convert();
        comment.setTime(conv.dateToFormatDTOString(new Time(System.currentTimeMillis())));
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        commentService.create(toCommentEntity(comment));
        return Constants.REDIRECT_VIEW + "/" + id + "/" + COMMENT_VIEW;
    }

    @GetMapping("{postId}/comments/edit/{id}")
    public String updatePostComment(
            @PathVariable(name = "postId") Long postId,
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @AuthenticationPrincipal UserPrincipal principal,
            Model model) {
        if (id <= 0 && postId <= 0) {
            throw new IllegalArgumentException();
        }
        if (principal.getId() != commentService.get(id).getperson().getId()) {
            return Constants.REDIRECT_VIEW + "/" + postId + "/" + COMMENT_VIEW;
        }
        model.addAttribute(COMMENT_ATTRIBUTE, toCommentDto(commentService.get(id)));
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return COMMENT_EDIT_VIEW;
    }

    @PostMapping("{postId}/comments/edit/{id}")
    public String updatePostComment(
            @PathVariable(name = "id") Long id,
            @PathVariable(name = "postId") Long postId,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = COMMENT_ATTRIBUTE) @Valid PostCommentDto comment,
            BindingResult bindingResult,
            @AuthenticationPrincipal UserPrincipal principal,
            Model model,
            RedirectAttributes redirectAttributes) {
        if (bindingResult.hasErrors()) {
            model.addAttribute(PAGE_ATTRIBUTE, page);
            return COMMENT_EDIT_VIEW;
        }
        convert conv = new convert();
        comment.setTime(conv.dateToFormatDTOString(new Time(System.currentTimeMillis())));
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        comment.setPersonId(principal.getId());
        comment.setPostId(postId);
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        commentService.update(id, toCommentEntity(comment));
        return Constants.REDIRECT_VIEW + "/" + postId + "/" + COMMENT_VIEW;
    }

    @PostMapping("{postId}/comments/delete/{id}")
    public String deletePostComment(
            @PathVariable(name = "id") Long id,
            @PathVariable(name = "postId") Long postId,
            @RequestParam(name = COMMENT_ATTRIBUTE, defaultValue = "0") int page,
            @AuthenticationPrincipal UserPrincipal principal,
            RedirectAttributes redirectAttributes) {
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        if (commentService.get(id).getperson().getId() == principal.getId())
            commentService.delete(id);
        return Constants.REDIRECT_VIEW + "/" + postId + "/" + COMMENT_VIEW;
    }
}
