package com.example.demo.posts.api;

import java.util.Map;

import org.modelmapper.ModelMapper;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.example.demo.core.convert;
import com.example.demo.core.api.PageAttributesMapper;
import com.example.demo.core.configuration.Constants;
import com.example.demo.posts.model.PostEntity;
import com.example.demo.posts.service.PostService;
import com.example.demo.users.service.UserService;

import jakarta.validation.Valid;

@Controller
@RequestMapping(PostController.URL)
public class PostController {
    public static final String URL = Constants.ADMIN_PREFIX + "/post";
    private static final String POST_VIEW = "post";
    private static final String POST_EDIT_VIEW = "post-edit";
    private static final String PAGE_ATTRIBUTE = "page";
    private static final String POST_ATTRIBUTE = "post";

    private final UserService userService;
    private final PostService postService;
    private final ModelMapper modelMapper;

    public PostController(PostService postService, UserService userService, ModelMapper modelMapper) {
        this.postService = postService;
        this.userService = userService;
        this.modelMapper = modelMapper;
    }

    private PostDto toDto(PostEntity entity) {
        var model = modelMapper.map(entity, PostDto.class);
        model.setPersonUsername(entity.getPerson().getUsername());
        convert time = new convert();
        model.setTime(time.dateToString(entity.getTime()));
        return model;
    }

    private PostEntity toEntity(PostDto dto) {
        final PostEntity entity = modelMapper.map(dto, PostEntity.class);
        entity.setPerson(userService.get(dto.getPersonId()));
        return entity;

    }

    @GetMapping
    public String getAll(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        final Map<String, Object> attributes = PageAttributesMapper.toAttributes(
                postService.getAll(page, Constants.DEFUALT_PAGE_SIZE), this::toDto);
        model.addAllAttributes(attributes);
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return POST_VIEW;
    }

    @GetMapping("/edit/")
    public String create(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        model.addAttribute(POST_ATTRIBUTE, new PostDto());
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return POST_EDIT_VIEW;
    }

    @PostMapping("/edit/")
    public String create(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = POST_ATTRIBUTE) @Valid PostDto post,
            BindingResult bindingResult,
            Model model,
            RedirectAttributes redirectAttributes) {
        if (bindingResult.hasErrors()) {
            model.addAttribute(PAGE_ATTRIBUTE, page);
            return POST_EDIT_VIEW;
        }
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        postService.create(toEntity(post));
        return Constants.REDIRECT_VIEW + URL;
    }

    @PostMapping("/like/{id}")
    public PostDto addLike(@PathVariable(name = "id") Long id) {
        return toDto(postService.addLike(id));
    }

    @DeleteMapping("/like/{id}")
    public PostDto removeLike(@PathVariable(name = "id") Long id) {
        return toDto(postService.removeLiked(id));
    }

    @PostMapping("/view/{id}")
    public PostDto addView(@PathVariable(name = "id") Long id) {
        return toDto(postService.addView(id));
    }

    @GetMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        model.addAttribute(POST_ATTRIBUTE, toDto(postService.get(id)));
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return POST_EDIT_VIEW;
    }

    @PostMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = POST_ATTRIBUTE) @Valid PostDto post,
            BindingResult bindingResult,
            Model model,
            RedirectAttributes redirectAttributes) {
        if (bindingResult.hasErrors()) {
            model.addAttribute(PAGE_ATTRIBUTE, page);
            return POST_EDIT_VIEW;
        }
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        postService.update(id, toEntity(post));
        return Constants.REDIRECT_VIEW + URL;
    }

    @PostMapping("/delete/{id}")
    public String delete(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            RedirectAttributes redirectAttributes) {
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        postService.delete(id);
        return Constants.REDIRECT_VIEW + URL;
    }
}
