package com.example.demo.users.api;

import java.util.Map;
import org.modelmapper.ModelMapper;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.example.demo.core.api.PageAttributesMapper;
import com.example.demo.core.configuration.Constants;
import com.example.demo.core.security.UserPrincipal;
import com.example.demo.users.model.FriendsEntity;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

import jakarta.validation.Valid;

@Controller
@RequestMapping(UserProfileController.URL)
public class UserProfileController {
    public static final String URL = "/profile";
    private static final String PROFILE_VIEW = "profile";
    private static final String FRIENDS_VIEW = "friends";
    private static final String PAGE_ATTRIBUTE = "page";
    private static final String USER_ATTRIBUTE = "user";

    private final UserService userService;
    private final ModelMapper modelMapper;

    public UserProfileController(UserService userService, ModelMapper modelMapper) {
        this.userService = userService;
        this.modelMapper = modelMapper;
    }

    private UserProfileDto toDto(UserEntity entity) {
        var model = modelMapper.map(entity, UserProfileDto.class);
        if (entity.getFriends() != null) {
            for (UserEntity user : entity.getFriends()) {
                model.addFriendId(user.getId());
            }
        }
        return model;

    }

    private FriendDto toDto(FriendsEntity entity) {
        var model = modelMapper.map(entity, FriendDto.class);
        return model;

    }

    private UserEntity toEntity(UserProfileDto dto) {
        return modelMapper.map(dto, UserEntity.class);
    }

    @GetMapping
    public String getProfile(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @AuthenticationPrincipal UserPrincipal principal,
            Model model) {
        var user = userService.get(principal.getId());
        model.addAttribute(USER_ATTRIBUTE, toDto(user));
        return PROFILE_VIEW;
    }

    @PostMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            @ModelAttribute(name = USER_ATTRIBUTE) @Valid UserProfileDto user,
            BindingResult bindingResult,
            Model model,
            RedirectAttributes redirectAttributes) {
        if (user.getUsername().length() < 4) {
            bindingResult.rejectValue("username", "profile:usernames", "Неправльный логин.");
            model.addAttribute(USER_ATTRIBUTE, user);
            return PROFILE_VIEW;
        }
        if (user.getEmail().length() < 4) {
            bindingResult.rejectValue("email", "profile:emails", "Неправильная почта.");
            model.addAttribute(USER_ATTRIBUTE, user);
            return PROFILE_VIEW;
        }
        if (user.getPassword().length() < 4 || user.getPassword().length() > 30) {
            bindingResult.rejectValue("password", "profile:passwords", "Неправильный пароль.");
            model.addAttribute(USER_ATTRIBUTE, user);
            return PROFILE_VIEW;
        }
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        userService.update(id, toEntity(user));
        return Constants.REDIRECT_VIEW + URL;
    }

    @GetMapping("/{id}/friends")
    public String getFriends(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @AuthenticationPrincipal UserPrincipal principal,
            Model model) {
        Map<String, Object> attributes = PageAttributesMapper
                .toAttributes(userService.getFriends(id, page, 5),
                        this::toDto);

        model.addAllAttributes(attributes);
        model.addAttribute("principal", toDto(userService.get(principal.getId())));
        return FRIENDS_VIEW;
    }

    @PostMapping("/{principalId}/friends/add/{id}")
    public String addFriend(
            @PathVariable(name = "id") Long id,
            @PathVariable(name = "principalId") Long principalId,
            @ModelAttribute(name = USER_ATTRIBUTE) @Valid UserProfileDto user,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @AuthenticationPrincipal UserPrincipal principal,
            Model model,
            RedirectAttributes redirectAttributes) {
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        userService.addFriend(principal.getId(), userService.get(id));
        return Constants.REDIRECT_VIEW + "/" + PROFILE_VIEW + "/" + principalId + "/" + FRIENDS_VIEW;
    }

    @PostMapping("/{principalId}/friends/remove/{id}")
    public String removeFriend(
            @PathVariable(name = "id") Long id,
            @PathVariable(name = "principalId") Long principalId,
            @ModelAttribute(name = USER_ATTRIBUTE) @Valid UserProfileDto user,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @AuthenticationPrincipal UserPrincipal principal,
            Model model,
            RedirectAttributes redirectAttributes) {
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        userService.removeFriend(principal.getId(), userService.get(id));
        return Constants.REDIRECT_VIEW + "/" + PROFILE_VIEW + "/" + principalId + "/" + FRIENDS_VIEW;
    }

}
