package com.example.demo.users.service;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import java.util.stream.StreamSupport;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import com.example.demo.core.configuration.Constants;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.core.security.UserPrincipal;
import com.example.demo.users.model.FriendsEntity;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.model.UserRole;
import com.example.demo.users.model.UserTop;
import com.example.demo.users.repository.UserRepository;

import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

@Service
public class UserService implements UserDetailsService {
    private final UserRepository repository;
    private final PasswordEncoder passwordEncoder;

    public UserService(UserRepository repository, PasswordEncoder passwordEncoder) {
        this.passwordEncoder = passwordEncoder;
        this.repository = repository;
    }

    private void checkLogin(Long id, String login) {
        final Optional<UserEntity> existsUser = repository.findByUsernameIgnoreCase(login);
        if (existsUser.isPresent() && !existsUser.get().getId().equals(id)
                && existsUser.get().getUsername() != "[deleted]") {
            throw new IllegalArgumentException(
                    String.format("User with login %s is already exists", login));
        }
    }

    @Transactional(readOnly = true)
    public List<UserEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public Page<UserEntity> getAll(int page, int size) {
        return repository.findAll(PageRequest.of(page, size));
    }

    @Transactional(readOnly = true)
    public Page<UserEntity> getByFriends(List<UserEntity> friends, int page, int size) {
        List<Long> ids = new ArrayList<Long>();
        for (UserEntity user : friends) {
            ids.add(user.getId());
        }
        return repository.findByFriends(ids, PageRequest.of(page, size));
    }

    @Transactional(readOnly = true)
    public Page<FriendsEntity> getFriends(Long id, int page, int size) {
        return repository.findFriendsById(id, PageRequest.of(page, size));
    }

    @Transactional(readOnly = true)
    public UserEntity get(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(UserEntity.class, id));
    }

    @Transactional(readOnly = true)
    public UserEntity getByLogin(String login) {
        return repository.findByUsernameIgnoreCase(login)
                .orElseThrow(() -> new IllegalArgumentException("Invalid login"));
    }

    @Transactional
    public UserEntity create(UserEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        checkLogin(null, entity.getUsername());
        final String password = Optional.ofNullable(entity.getPassword()).orElse("");
        entity.setPassword(
                passwordEncoder.encode(
                        StringUtils.hasText(password.strip()) ? password : Constants.DEFAULT_PASSWORD));
        entity.setRole(Optional.ofNullable(entity.getRole()).orElse(UserRole.USER));
        return repository.save(entity);
    }

    @Transactional
    public UserEntity addFriend(Long id, UserEntity entity) {
        final UserEntity existsEntity = get(id);
        if (existsEntity.getFriends() == null)
            existsEntity.setFriends(new ArrayList<UserEntity>());
        existsEntity.getFriends().add(entity);
        repository.save(existsEntity);
        return existsEntity;
    }

    @Transactional
    public UserEntity removeFriend(Long id, UserEntity entity) {
        final UserEntity existsEntity = get(id);

        existsEntity.getFriends().removeIf(friend -> friend.getId() == entity.getId());
        repository.save(existsEntity);
        return existsEntity;
    }

    @Override
    @Transactional(readOnly = true)
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        final UserEntity existsUser = getByLogin(username);
        if (username == "[deleted]")
            return null;
        return new UserPrincipal(existsUser);
    }

    @Transactional
    public UserEntity update(long id, UserEntity entity) {
        final UserEntity existsEntity = get(id);
        checkLogin(id, entity.getUsername());
        existsEntity.setUsername(entity.getUsername());
        existsEntity.setEmail(entity.getEmail());
        if ((entity.getPassword() != null) && (entity.getPassword().length() < 30))
            existsEntity.setPassword(
                    passwordEncoder.encode(
                            StringUtils.hasText(entity.getPassword().strip()) ? entity.getPassword()
                                    : Constants.DEFAULT_PASSWORD));
        repository.save(existsEntity);
        return existsEntity;
    }

    @Transactional
    public UserEntity delete(long id) {
        final UserEntity existsEntity = get(id);
        existsEntity.setUsername("[deleted]");
        existsEntity.setEmail("[deleted]");
        existsEntity.setPassword("[deleted]");
        existsEntity.setFriends(null);
        repository.save(existsEntity);
        return existsEntity;
    }

    @Transactional(readOnly = true)
    public Page<UserTop> getTop(int page, int size) {
        return repository.getFiveWithTopFriends(PageRequest.of(page, size));
    }
}
